@@ -1,9 +1,10 @@
-__version__ = '0.5'
+__version__ = '0.7.0'
 
 __all__ = ['Label', 'Instr', 'SetLineno', 'Bytecode',
            'ConcreteInstr', 'ConcreteBytecode',
-           'ControlFlowGraph']
+           'ControlFlowGraph', 'CompilerFlags']
 
+from bytecode.flags import CompilerFlags
 from bytecode.instr import (UNSET, Label, SetLineno, Instr, CellVar, FreeVar,   # noqa
                             Compare)
 from bytecode.bytecode import BaseBytecode, _InstrList, Bytecode   # noqa
