@@ -30,9 +30,13 @@ class ConcreteInstr(Instr):
     It has a read-only size attribute.
     """
 
-    __slots__ = ('_size',)
+    __slots__ = ('_size', '_extended_args')
 
-    def __init__(self, name, arg=UNSET, *, lineno=None):
+    def __init__(self, name, arg=UNSET, *, lineno=None, extended_args=None):
+        # Allow to remember a potentially meaningless EXTENDED_ARG emitted by
+        # Python to properly compute the size and avoid messing up the jump
+        # targets
+        self._extended_args = extended_args
         self._set(name, arg, lineno)
 
     def _check_arg(self, name, opcode, arg):
@@ -53,12 +57,16 @@ class ConcreteInstr(Instr):
                 while arg > 0xff:
                     size += 2
                     arg >>= 8
+            if self._extended_args is not None:
+                size = 2 + 2 * self._extended_args
         else:
             size = 1
             if arg is not UNSET:
                 size += 2
                 if arg > 0xffff:
                     size += 3
+                if self._extended_args is not None:
+                    size = 1 + 3 * self._extended_args
         self._size = size
 
     @property
@@ -86,6 +94,10 @@ class ConcreteInstr(Instr):
                 arg >>= 8
                 b[:0] = [_opcode.EXTENDED_ARG, arg & 0xff]
 
+            if self._extended_args:
+                while len(b) < self._size:
+                    b[:0] = [_opcode.EXTENDED_ARG, 0x00]
+
             return bytes(b)
     else:
         def assemble(self):
@@ -94,11 +106,17 @@ class ConcreteInstr(Instr):
 
             arg = self._arg
             if arg > 0xffff:
-                return struct.pack('<BHBH',
-                                   _opcode.EXTENDED_ARG, arg >> 16,
-                                   self._opcode, arg & 0xffff)
+                b = struct.pack('<BHBH',
+                                _opcode.EXTENDED_ARG, arg >> 16,
+                                self._opcode, arg & 0xffff)
             else:
-                return struct.pack('<BH', self._opcode, arg)
+                b = struct.pack('<BH', self._opcode, arg)
+
+            if self._extended_args:
+                while len(b) < self._size:
+                    b = struct.pack('<BH', _opcode.EXTENDED_ARG, 0) + b
+
+            return b
 
     @classmethod
     def disassemble(cls, lineno, code, offset):
@@ -116,23 +134,28 @@ class ConcreteInstr(Instr):
 
 class ConcreteBytecode(_bytecode.BaseBytecode, list):
 
-    def __init__(self):
+    def __init__(self, instructions=(), *, consts=(), names=(), varnames=()):
         super().__init__()
-        self.consts = []
-        self.names = []
-        self.varnames = []
+        self.consts = list(consts)
+        self.names = list(names)
+        self.varnames = list(varnames)
+        for instr in instructions:
+            self._check_instr(instr)
+        self.extend(instructions)
 
     def __iter__(self):
         instructions = super().__iter__()
         for instr in instructions:
-            if not isinstance(instr, (ConcreteInstr, SetLineno)):
-                raise ValueError("ConcreteBytecode must only contain "
-                                 "ConcreteInstr and SetLineno objects, "
-                                 "but %s was found"
-                                 % instr.__class__.__name__)
-
+            self._check_instr(instr)
             yield instr
 
+    def _check_instr(self, instr):
+        if not isinstance(instr, (ConcreteInstr, SetLineno)):
+            raise ValueError("ConcreteBytecode must only contain "
+                             "ConcreteInstr and SetLineno objects, "
+                             "but %s was found"
+                             % type(instr).__name__)
+
     def __repr__(self):
         return '<ConcreteBytecode instr#=%s>' % len(self)
 
@@ -170,13 +193,20 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
             offset += instr.size
 
         # replace jump targets with blocks
+        # HINT : in some cases Python generate useless EXTENDED_ARG opcode
+        # with a value of zero. Such opcodes do not increases the size of the
+        # following opcode the way a normal EXTENDED_ARG does. As a
+        # consequence, they need to be tracked manually as otherwise the
+        # offsets in jump targets can end up being wrong.
         if not extended_arg:
+            nb_extended_args = 0
             extended_arg = None
             index = 0
             while index < len(instructions):
                 instr = instructions[index]
 
                 if instr.name == 'EXTENDED_ARG':
+                    nb_extended_args += 1
                     if extended_arg is not None:
                         if not _WORDCODE:
                             raise ValueError("EXTENDED_ARG followed "
@@ -184,6 +214,7 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
                         extended_arg = (extended_arg << 8) + instr.arg
                     else:
                         extended_arg = instr.arg
+
                     del instructions[index]
                     continue
 
@@ -194,8 +225,10 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
                         arg = (extended_arg << 16) + instr.arg
                     extended_arg = None
 
-                    instr = ConcreteInstr(instr.name, arg, lineno=instr.lineno)
+                    instr = ConcreteInstr(instr.name, arg, lineno=instr.lineno,
+                                          extended_args=nb_extended_args)
                     instructions[index] = instr
+                    nb_extended_args = 0
 
                 index += 1
 
@@ -208,7 +241,6 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
         bytecode.flags = code.co_flags
         bytecode.argcount = code.co_argcount
         bytecode.kwonlyargcount = code.co_kwonlyargcount
-        bytecode._stacksize = code.co_stacksize
         bytecode.first_lineno = code.co_firstlineno
         bytecode.names = list(code.co_names)
         bytecode.consts = list(code.co_consts)
@@ -279,16 +311,21 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
 
         return b''.join(lnotab)
 
+    def compute_stacksize(self):
+        bytecode = self.to_bytecode()
+        cfg = _bytecode.ControlFlowGraph.from_bytecode(bytecode)
+        return cfg.compute_stacksize()
+
     def to_code(self):
         code_str, linenos = self._assemble_code()
         lnotab = self._assemble_lnotab(self.first_lineno, linenos)
         nlocals = len(self.varnames)
+        stacksize = self.compute_stacksize()
         return types.CodeType(self.argcount,
                               self.kwonlyargcount,
                               nlocals,
-                              # FIXME: compute stack size
-                              self._stacksize,
-                              self.flags,
+                              stacksize,
+                              int(self.flags),
                               code_str,
                               tuple(self.consts),
                               tuple(self.names),
@@ -380,6 +417,10 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
 
 class _ConvertBytecodeToConcrete:
 
+    # Default number of passes of compute_jumps() before giving up.  Refer to
+    # assemble_jump_offsets() in compile.c for background.
+    _compute_jumps_passes = 10
+
     def __init__(self, code):
         assert isinstance(code, _bytecode.Bytecode)
         self.bytecode = code
@@ -487,7 +528,10 @@ class _ConvertBytecodeToConcrete:
 
         return modified
 
-    def to_concrete_bytecode(self):
+    def to_concrete_bytecode(self, compute_jumps_passes=None):
+        if compute_jumps_passes is None:
+            compute_jumps_passes = self._compute_jumps_passes
+
         first_const = self.bytecode.docstring
         if first_const is not UNSET:
             self.add_const(first_const)
@@ -495,24 +539,25 @@ class _ConvertBytecodeToConcrete:
         self.varnames.extend(self.bytecode.argnames)
 
         self.concrete_instructions()
-        modified = self.compute_jumps()
-        if modified:
+        for pas in range(0, compute_jumps_passes):
             modified = self.compute_jumps()
-            if modified:
-                raise RuntimeError("compute_jumps() must not modify jumps "
-                                   "at the second iteration")
+            if not modified:
+                break
+        else:
+            raise RuntimeError("compute_jumps() failed to converge after"
+                               " %d passes"
+                               % (pas + 1))
 
         consts = [None] * len(self.consts)
         for item, index in self.consts.items():
             # const_key(value)[1] is value: see const_key() function
             consts[index] = item[1]
 
-        concrete = ConcreteBytecode()
+        concrete = ConcreteBytecode(
+            self.instructions,
+            consts=consts,
+            names=self.names,
+            varnames=self.varnames
+        )
         concrete._copy_attr_from(self.bytecode)
-        concrete.consts = consts
-        concrete.names = self.names
-        concrete.varnames = self.varnames
-
-        # copy instructions
-        concrete[:] = self.instructions
         return concrete
