@@ -3,6 +3,7 @@ import opcode
 import sys
 import types
 import unittest
+import textwrap
 from bytecode import (UNSET, Label, Instr, SetLineno, Bytecode,
                       CellVar, FreeVar,
                       ConcreteInstr, ConcreteBytecode)
@@ -35,6 +36,13 @@ class ConcreteInstrTests(TestCase):
         instr = ConcreteInstr("LOAD_CONST", 2147483647)
         self.assertEqual(instr.arg, 2147483647)
 
+        # test meaningless extended args
+        instr = ConcreteInstr('LOAD_FAST', 8, lineno=3, extended_args=1)
+        self.assertEqual(instr.name, 'LOAD_FAST')
+        self.assertEqual(instr.arg, 8)
+        self.assertEqual(instr.lineno, 3)
+        self.assertEqual(instr.size, 4)
+
     def test_attr(self):
         instr = ConcreteInstr("LOAD_CONST", 5, lineno=12)
         self.assertEqual(instr.name, 'LOAD_CONST')
@@ -117,10 +125,13 @@ class ConcreteInstrTests(TestCase):
         instr = ConcreteInstr.disassemble(2, code, 2 if WORDCODE else 1)
         self.assertEqual(instr, ConcreteInstr("LOAD_CONST", 3, lineno=2))
 
-        code = b'\x90\x12\x904\x90\xabd\xcd' if WORDCODE else b'\x904\x12d\xcd\xab'
+        code = (b'\x90\x12\x904\x90\xabd\xcd' if WORDCODE else
+                b'\x904\x12d\xcd\xab')
+
         instr = ConcreteInstr.disassemble(3, code, 0)
         self.assertEqual(instr,
-                         ConcreteInstr('EXTENDED_ARG', 0x12 if WORDCODE else 0x1234, lineno=3))
+                         ConcreteInstr('EXTENDED_ARG',
+                                       0x12 if WORDCODE else 0x1234, lineno=3))
 
     def test_assemble(self):
         instr = ConcreteInstr("NOP")
@@ -131,8 +142,14 @@ class ConcreteInstrTests(TestCase):
                          b'd\x03' if WORDCODE else b'd\x03\x00')
 
         instr = ConcreteInstr("LOAD_CONST", 0x1234abcd)
-        self.assertEqual(instr.assemble(
-        ), b'\x90\x12\x904\x90\xabd\xcd' if WORDCODE else b'\x904\x12d\xcd\xab')
+        self.assertEqual(instr.assemble(),
+                         (b'\x90\x12\x904\x90\xabd\xcd' if WORDCODE else
+                          b'\x904\x12d\xcd\xab'))
+
+        instr = ConcreteInstr("LOAD_CONST", 3, extended_args=1)
+        self.assertEqual(instr.assemble(),
+                         (b'\x90\x00d\x03' if WORDCODE else
+                          b'\x90\x00\x00d\x03\x00'))
 
     def test_get_jump_target(self):
         jump_abs = ConcreteInstr("JUMP_ABSOLUTE", 3)
@@ -159,11 +176,13 @@ class ConcreteBytecodeTests(TestCase):
                               ConcreteInstr('RETURN_VALUE', lineno=1)])
         # FIXME: test other attributes
 
-    def test_iter_invalid_types(self):
+    def test_invalid_types(self):
         code = ConcreteBytecode()
         code.append(Label())
         with self.assertRaises(ValueError):
             list(code)
+        with self.assertRaises(ValueError):
+            ConcreteBytecode([Label()])
 
     def test_to_code_lnotab(self):
         # x = 7
@@ -195,21 +214,23 @@ class ConcreteBytecodeTests(TestCase):
         self.assertEqual(code.co_code, expected)
         self.assertEqual(code.co_firstlineno, 3)
         self.assertEqual(
-            code.co_lnotab, b'\x04\x01\x04\x01' if WORDCODE else b'\x06\x01\x06\x01')
+            code.co_lnotab,
+            b'\x04\x01\x04\x01' if WORDCODE else b'\x06\x01\x06\x01')
 
     def test_negative_lnotab(self):
         # x = 7
         # y = 8
-        concrete = ConcreteBytecode()
+        concrete = ConcreteBytecode([
+            ConcreteInstr("LOAD_CONST", 0),
+            ConcreteInstr("STORE_NAME", 0),
+            # line number goes backward!
+            SetLineno(2),
+            ConcreteInstr("LOAD_CONST", 1),
+            ConcreteInstr("STORE_NAME", 1)
+        ])
         concrete.consts = [7, 8]
         concrete.names = ['x', 'y']
         concrete.first_lineno = 5
-        concrete.extend([ConcreteInstr("LOAD_CONST", 0),
-                         ConcreteInstr("STORE_NAME", 0),
-                         # line number goes backward!
-                         SetLineno(2),
-                         ConcreteInstr("LOAD_CONST", 1),
-                         ConcreteInstr("STORE_NAME", 1)])
 
         if sys.version_info >= (3, 6):
             code = concrete.to_code()
@@ -319,8 +340,10 @@ class ConcreteBytecodeTests(TestCase):
         self.assertEqual(bytecode.freevars, ['__class__'])
         self.assertEqual(bytecode.cellvars, ['__class__'])
         self.assertEqual(list(bytecode),
-                         [Instr('LOAD_CLASSDEREF', FreeVar('__class__'), lineno=1),
-                          Instr('STORE_DEREF', FreeVar('__class__'), lineno=1)])
+                         [Instr('LOAD_CLASSDEREF', FreeVar('__class__'),
+                                lineno=1),
+                          Instr('STORE_DEREF', FreeVar('__class__'), lineno=1)]
+                         )
 
         concrete = bytecode.to_concrete_bytecode()
         self.assertEqual(concrete.freevars, ['__class__'])
@@ -333,14 +356,16 @@ class ConcreteBytecodeTests(TestCase):
         self.assertEqual(code.co_freevars, ('__class__',))
         self.assertEqual(code.co_cellvars, ('__class__',))
         self.assertEqual(
-            code.co_code, b'\x94\x01\x89\x01' if WORDCODE else b'\x94\x01\x00\x89\x01\x00')
+            code.co_code,
+            b'\x94\x01\x89\x01' if WORDCODE else b'\x94\x01\x00\x89\x01\x00')
 
 
 class ConcreteFromCodeTests(TestCase):
 
     def test_extended_arg(self):
         # Create a code object from arbitrary bytecode
-        co_code = b'\x90\x12\x904\x90\xabd\xcd' if WORDCODE else b'\x904\x12d\xcd\xab'
+        co_code = (b'\x90\x12\x904\x90\xabd\xcd' if WORDCODE else
+                   b'\x904\x12d\xcd\xab')
         code = get_code('x=1')
         code = types.CodeType(code.co_argcount,
                               code.co_kwonlyargcount,
@@ -361,7 +386,8 @@ class ConcreteFromCodeTests(TestCase):
         # without EXTENDED_ARG opcode
         bytecode = ConcreteBytecode.from_code(code)
         self.assertListEqual(list(bytecode),
-                             [ConcreteInstr("LOAD_CONST", 0x1234abcd, lineno=1)])
+                             [ConcreteInstr("LOAD_CONST", 0x1234abcd,
+                                            lineno=1)])
 
         # with EXTENDED_ARG opcode
         bytecode = ConcreteBytecode.from_code(code, extended_arg=True)
@@ -476,6 +502,68 @@ class BytecodeToConcreteTests(TestCase):
         self.assertListEqual(concrete.names, ['test', 'x'])
         self.assertListEqual(concrete.varnames, [])
 
+    def test_label3(self):
+        """
+        CPython generates useless EXTENDED_ARG 0 in some cases. We need to
+        properly track them as otherwise we can end up with broken offset for
+        jumps.
+        """
+        source = """
+            def func(x):
+                if x == 1:
+                    return x + 0
+                elif x == 2:
+                    return x + 1
+                elif x == 3:
+                    return x + 2
+                elif x == 4:
+                    return x + 3
+                elif x == 5:
+                    return x + 4
+                elif x == 6:
+                    return x + 5
+                elif x == 7:
+                    return x + 6
+                elif x == 8:
+                    return x + 7
+                elif x == 9:
+                    return x + 8
+                elif x == 10:
+                    return x + 9
+                elif x == 11:
+                    return x + 10
+                elif x == 12:
+                    return x + 11
+                elif x == 13:
+                    return x + 12
+                elif x == 14:
+                    return x + 13
+                elif x == 15:
+                    return x + 14
+                elif x == 16:
+                    return x + 15
+                elif x == 17:
+                    return x + 16
+                return -1
+        """
+        code = get_code(source, function=True)
+        bcode = Bytecode.from_code(code)
+        concrete = bcode.to_concrete_bytecode()
+        self.assertIsInstance(concrete, ConcreteBytecode)
+
+        # Ensure that we do not generate broken code
+        loc = {}
+        exec(textwrap.dedent(source), loc)
+        func = loc['func']
+        func.__code__ = bcode.to_code()
+        for i, x in enumerate(range(1, 18)):
+            self.assertEqual(func(x), x + i)
+        self.assertEqual(func(18), -1)
+
+        # Ensure that we properly round trip in such cases
+        self.assertEqual(ConcreteBytecode.from_code(code).to_code().co_code,
+                         code.co_code)
+
     def test_setlineno(self):
         # x = 7
         # y = 8
@@ -522,13 +610,15 @@ class BytecodeToConcreteTests(TestCase):
         nb_nop = 2**16
         code = Bytecode([Instr("JUMP_ABSOLUTE", label),
                          BigInstr(nb_nop),
-                         label])
+                         label,
+                         Instr('LOAD_CONST', None),
+                         Instr('RETURN_VALUE')])
 
         code_obj = code.to_code()
         if WORDCODE:
-            expected = (b'\x90\x01\x90\x00q\x06' + NOP * nb_nop)
+            expected = b'\x90\x01\x90\x00q\x06' + NOP * nb_nop + b'd\x00S\x00'
         else:
-            expected = (b'\x90\x01\x00q\x06\x00' + NOP * nb_nop)
+            expected = b'\x90\x01\x00q\x06\x00' + NOP * nb_nop + b'd\x00\x00S'
         self.assertEqual(code_obj.co_code, expected)
 
     def test_jumps(self):
@@ -596,6 +686,113 @@ class BytecodeToConcreteTests(TestCase):
         code.extend([ConcreteInstr("LOAD_DEREF", 0, lineno=1),
                      ConcreteInstr("LOAD_DEREF", 1, lineno=1)])
 
+    def test_compute_jumps_convergence(self):
+        # Consider the following sequence of instructions:
+        #
+        #     JUMP_ABSOLUTE Label1
+        #     JUMP_ABSOLUTE Label2
+        #     ...126 instructions...
+        #   Label1:                 Offset 254 on first pass, 256 second pass
+        #     NOP
+        #     ... many more instructions ...
+        #   Label2:                 Offset > 256 on first pass
+        #
+        # On first pass of compute_jumps(), Label2 will be at address 254, so
+        # that value encodes into the single byte arg of JUMP_ABSOLUTE.
+        #
+        # On second pass compute_jumps() the instr at Label1 will have offset
+        # of 256 so will also be given an EXTENDED_ARG.
+        #
+        # Thus we need to make an additional pass.  This test only verifies
+        # case where 2 passes is insufficient but three is enough.
+
+        if not WORDCODE:
+            # Could be done pre-WORDCODE, but that requires 2**16 bytes of
+            # code.
+            return
+
+        # Create code from comment above.
+        code = Bytecode()
+        label1 = Label()
+        label2 = Label()
+        nop = 'UNARY_POSITIVE'   # don't use NOP, dis.stack_effect will raise
+        code.append(Instr('JUMP_ABSOLUTE', label1))
+        code.append(Instr('JUMP_ABSOLUTE', label2))
+        for x in range(4, 254, 2):
+            code.append(Instr(nop))
+        code.append(label1)
+        code.append(Instr(nop))
+        for x in range(256, 300, 2):
+            code.append(Instr(nop))
+        code.append(label2)
+        code.append(Instr(nop))
+
+        # This should pass by default.
+        code.to_code()
+
+        # Try with max of two passes:  it should raise
+        with self.assertRaises(RuntimeError):
+            code.to_code(compute_jumps_passes=2)
+
+    def test_extreme_compute_jumps_convergence(self):
+        """Test of compute_jumps() requiring absurd number of passes.
+
+        NOTE:  This test also serves to demonstrate that there is no worst
+        case: the number of passes can be unlimited (or, actually, limited by
+        the size of the provided code).
+
+        This is an extension of test_compute_jumps_convergence.  Instead of
+        two jumps, where the earlier gets extended after the latter, we
+        instead generate a series of many jumps.  Each pass of compute_jumps()
+        extends one more instruction, which in turn causes the one behind it
+        to be extended on the next pass.
+        """
+        if not WORDCODE:
+            return
+
+        # N: the number of unextended instructions that can be squeezed into a
+        # set of bytes adressable by the arg of an unextended instruction.
+        # The answer is "128", but here's how we arrive at it (and it also
+        # hints at how to make this work for pre-WORDCODE).
+        max_unextended_offset = 1 << 8
+        unextended_branch_instr_size = 2
+        N = max_unextended_offset // unextended_branch_instr_size
+
+        nop = 'UNARY_POSITIVE'   # don't use NOP, dis.stack_effect will raise
+
+        # The number of jumps will be equal to the number of labels.  The
+        # number of passes of compute_jumps() required will be one greater
+        # than this.
+        labels = [Label() for x in range(0, 3 * N)]
+
+        code = Bytecode()
+        code.extend(Instr('JUMP_FORWARD', labels[len(labels) - x - 1])
+                    for x in range(0, len(labels)))
+        end_of_jumps = len(code)
+        code.extend(Instr(nop) for x in range(0, N))
+
+        # Now insert the labels.  The first is N instructions (i.e. 256
+        # bytes) after the last jump.  Then they proceed to earlier positions
+        # 4 bytes at a time.  While the targets are in the range of the nop
+        # instructions, 4 bytes is two instructions.  When the targets are in
+        # the range of JUMP_FORWARD instructions we have to allow for the fact
+        # that the instructions will have been extended to four bytes each, so
+        # working backwards 4 bytes per label means just one instruction per
+        # label.
+        offset = end_of_jumps + N
+        for l in range(0, len(labels)):
+            code.insert(offset, labels[l])
+            if offset <= end_of_jumps:
+                offset -= 1
+            else:
+                offset -= 2
+
+        code.insert(0, Instr("LOAD_CONST", 0))
+        del end_of_jumps
+        code.append(Instr('RETURN_VALUE'))
+
+        code.to_code(compute_jumps_passes=(len(labels) + 1))
+
 
 if __name__ == "__main__":
     unittest.main()
